@@ -42,6 +42,7 @@ import email
 import poplib
 import imaplib
 import re
+import select
 
 try:
     # do we have a recent pykerberos?
@@ -119,6 +120,7 @@ EAI_FAIL = getattr(socket, 'EAI_FAIL', NO_OBJ)
 class POP3initMixIn(object):
     '''Mix-In class to do POP3 non-SSL initialization.
     '''
+    SSL = False
     def _connect(self):
         self.log.trace()
         try:
@@ -144,6 +146,7 @@ class Py24POP3SSLinitMixIn(object):
     '''Mix-In class to do POP3 over SSL initialization with Python 2.4's
     poplib.POP3_SSL class.
     '''
+    SSL = True
     def _connect(self):
         self.log.trace()
         if not hasattr(socket, 'ssl'):
@@ -191,6 +194,7 @@ class Py23POP3SSLinitMixIn(object):
     '''Mix-In class to do POP3 over SSL initialization with custom-implemented
     code to support SSL with Python 2.3's poplib.POP3 class.
     '''
+    SSL = True
     def _connect(self):
         self.log.trace()
         if not hasattr(socket, 'ssl'):
@@ -237,6 +241,7 @@ class Py23POP3SSLinitMixIn(object):
 class IMAPinitMixIn(object):
     '''Mix-In class to do IMAP non-SSL initialization.
     '''
+    SSL = False
     def _connect(self):
         self.log.trace()
         try:
@@ -259,6 +264,7 @@ class IMAPinitMixIn(object):
 class IMAPSSLinitMixIn(object):
     '''Mix-In class to do IMAP over SSL initialization.
     '''
+    SSL = True
     def _connect(self):
         self.log.trace()
         if not hasattr(socket, 'ssl'):
@@ -330,7 +336,7 @@ class RetrieverSkeleton(ConfigurableBase):
                      - name - parameter name
                      - type - a type function to compare the parameter value
                      against (i.e. str, int, bool)
-                     - default - optional default value.  If not preseent, the
+                     - default - optional default value.  If not present, the
                      parameter is required.
 
       __str__(self) - return a simple string representing the class instance.
@@ -372,7 +378,6 @@ class RetrieverSkeleton(ConfigurableBase):
       initialize(self, options)
       checkconf(self)
     '''
-
     def __init__(self, **args):
         self.headercache = {}
         self.deleted = {}
@@ -381,6 +386,8 @@ class RetrieverSkeleton(ConfigurableBase):
         self.__initialized = False
         self.gotmsglist = False
         self._clear_state()
+        self.conn = None
+        self.supports_idle = False
         ConfigurableBase.__init__(self, **args)
 
     def _clear_state(self):
@@ -432,15 +439,16 @@ class RetrieverSkeleton(ConfigurableBase):
                 or (isinstance(mailbox, (str, unicode)) and mailbox)), (
             'bad mailbox %s (%s)' % (mailbox, type(mailbox))
         )
-        if isinstance(mailbox, str):
-            mailbox = mailbox.decode('utf-8')
         filename = self.oldmail_filename
-        if mailbox is None:
-            # No mailbox (POP), use above with no extension
-            pass
-        else:
+        if mailbox is not None:
+            if isinstance(mailbox, str):
+                mailbox = mailbox.decode('utf-8')
+            mailbox = re.sub(STRIP_CHAR_RE, '.', mailbox)
+            mailbox = mailbox.encode('utf-8')
             # Use oldmail file per IMAP folder
-            filename += '-' + re.sub(STRIP_CHAR_RE, '.', mailbox)
+            filename += '-' + mailbox
+        # else:
+            # mailbox is None, is POP, just use filename
         return filename
 
     def oldmail_exists(self, mailbox):
@@ -472,6 +480,13 @@ class RetrieverSkeleton(ConfigurableBase):
                 continue
             try:
                 (msgid, timestamp) = line.split('\0', 1)
+                if msgid.count('/') == 2:
+                    # Was pre-4.22.0 file format, which includes the
+                    # mailbox name in the msgid, in the format
+                    # 'uidvalidity/mailbox/serveruid'.
+                    # Strip it out.
+                    fields = msgid.split('/')
+                    msgid = '/'.join([fields[0], fields[2]])
                 self.oldmail[msgid] = int(timestamp)
             except ValueError:
                 # malformed
@@ -621,6 +636,10 @@ class POP3RetrieverBase(RetrieverSkeleton):
             for (i, line) in enumerate(msglist):
                 try:
                     (msgnum, msgid) = line.split(None, 1)
+                    # Don't allow / in UIDs we store, as we look for that to 
+                    # detect old-style oldmail files.  Shouldn't occur in POP3
+                    # anyway.
+                    msgid = msgid.replace('/', '-')
                 except ValueError:
                     # Line didn't contain two tokens.  Server is broken.
                     raise getmailOperationError(
@@ -694,7 +713,7 @@ class POP3RetrieverBase(RetrieverSkeleton):
             response, lines, octets = self.conn.retr(msgnum)
             self.log.debug('RETR response "%s", %d octets'
                            % (response, octets) + os.linesep)
-            msg = Message(fromlines=lines)
+            msg = Message(fromlines=lines+[''])
             return msg
         except poplib.error_proto, o:
             raise getmailRetrievalError(
@@ -716,8 +735,8 @@ class POP3RetrieverBase(RetrieverSkeleton):
         # Handle password
         if self.conf.get('password', None) is None:
             self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'], 'pop3', 
-                self.log
+                self, self.conf['username'], self.conf['server'],
+                self.received_with, self.log
             )
         RetrieverSkeleton.initialize(self, options)
         try:
@@ -743,6 +762,8 @@ class POP3RetrieverBase(RetrieverSkeleton):
     def abort(self):
         self.log.trace()
         RetrieverSkeleton.abort(self)
+        if not self.conn:
+            return
         try:
             self.conn.rset()
             self.conn.quit()
@@ -753,7 +774,7 @@ class POP3RetrieverBase(RetrieverSkeleton):
     def quit(self):
         RetrieverSkeleton.quit(self)
         self.log.trace()
-        if not getattr(self, 'conn', None):
+        if not self.conn:
             return
         try:
             self.conn.quit()
@@ -1005,7 +1026,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
 
         self.log.debug('selecting mailbox "%s"' % mailbox + os.linesep)
         try:
-            if self.app_options['delete'] or self.app_options['delete_after']:
+            if (self.app_options['delete'] or self.app_options['delete_after'] 
+                    or self.app_options['delete_bigger_than']):
                 read_only = False
             else:
                 read_only = True
@@ -1045,9 +1067,11 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 )
                 for line in response:
                     r = self._parse_imapattrresponse(line)
-                    msgid = (
-                        '%s/%s' % (self.uidvalidity, r['uid'])
-                    )
+                    # Don't allow / in UIDs we store, as we look for that to 
+                    # detect old-style oldmail files.  Can occur with IMAP, at 
+                    # least with some servers.
+                    uid = r['uid'].replace('/', '-')
+                    msgid = '%s/%s' % (self.uidvalidity, uid)
                     self._mboxuids[msgid] = r['uid']
                     self._mboxuidorder.append(msgid)
                     self.msgnum_by_msgid[msgid] = None
@@ -1134,18 +1158,77 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 # response[0] is None instead of a message tuple
                 raise getmailRetrievalError('failed to retrieve msgid %s' 
                                             % msgid)
+
+            # record mailbox retrieved from in a header
+            msg.add_header('X-getmail-retrieved-from-mailbox', 
+                           self.mailbox_selected)
+
+            # google extensions: apply labels, etc
+            if 'X-GM-EXT-1' in self.conn.capabilities:
+                metadata = self._getgmailmetadata(uid, msg)
+                for (header, value) in metadata.items():
+                    msg.add_header(header, value)
+
             return msg
 
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
 
+    def _getgmailmetadata(self, uid, msg):
+        """
+        Add Gmail labels and other metadata which Google exposes through an
+        IMAP extension to headers in the message.
+        
+        See https://developers.google.com/google-apps/gmail/imap_extensions
+        """
+        try:
+            # ['976 (X-GM-THRID 1410134259107225671 X-GM-MSGID '
+            #   '1410134259107225671 X-GM-LABELS (labels space '
+            #   'separated) UID 167669)']
+            response = self._parse_imapuidcmdresponse('FETCH', uid,
+                '(X-GM-LABELS X-GM-THRID X-GM-MSGID)')
+        except imaplib.IMAP4.error, o:
+            self.log.warning('Could not fetch google imap extensions: %s' % o)
+            return {}
+
+        if not response:
+            return {}
+            
+        ext = re.search(
+            'X-GM-THRID (?P<THRID>\d+) X-GM-MSGID (?P<MSGID>\d+)'
+            ' X-GM-LABELS \((?P<LABELS>.*)\) UID',
+            response[0]
+        )
+        if not ext:
+            self.log.warning(
+                'Could not parse google imap extensions. Server said: %s'
+                % repr(response)
+            )
+            return {}
+
+        results = ext.groupdict()
+        metadata = {}
+        for item in ('LABELS', 'THRID', 'MSGID'):
+            if item in results and results[item]:
+                metadata['X-GMAIL-%s' % item] = results[item]
+
+        return metadata
+
     def _getmsgbyid(self, msgid):
         self.log.trace()
-        return self._getmsgpartbyid(msgid, '(BODY.PEEK[])')
+        if self.conf.get('use_peek', True):
+            part = '(BODY.PEEK[])'
+        else:
+            part = '(RFC822)'
+        return self._getmsgpartbyid(msgid, part)
 
     def _getheaderbyid(self, msgid):
         self.log.trace()
-        return self._getmsgpartbyid(msgid, '(BODY.PEEK[header])')
+        if self.conf.get('use_peek', True):
+            part = '(BODY.PEEK[header])'
+        else:
+            part = '(RFC822[header])'
+        return self._getmsgpartbyid(msgid, part)
 
     def initialize(self, options):
         self.log.trace()
@@ -1154,8 +1237,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         if (self.conf.get('password', None) is None
                 and not (HAVE_KERBEROS_GSS and self.conf['use_kerberos'])):
             self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'], 'imap',
-                self.log
+                self, self.conf['username'], self.conf['server'], 
+                self.received_with, self.log
             )
             
         RetrieverSkeleton.initialize(self, options)
@@ -1198,7 +1281,11 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                                    + os.linesep)
                     del self.oldmail[msgid]
             """
-            
+
+            if 'IDLE' in self.conn.capabilities:
+                self.supports_idle = True
+                imaplib.Commands['IDLE'] = ('AUTH', 'SELECTED')
+
             if self.mailboxes == ('ALL', ):
                 # Special value meaning all mailboxes in account
                 self.mailboxes = tuple(self.list_mailboxes())
@@ -1209,15 +1296,83 @@ class IMAPRetrieverBase(RetrieverSkeleton):
     def abort(self):
         self.log.trace()
         RetrieverSkeleton.abort(self)
+        if not self.conn:
+            return
         try:
             self.quit()
         except (imaplib.IMAP4.error, socket.error), o:
             pass
         self.conn = None
 
+    def go_idle(self, folder, timeout=29*60):
+        """Initiates IMAP's IDLE mode if the server supports it
+
+        Waits until state of current mailbox changes, and then returns. Returns
+        True if the connection still seems to be up, False otherwise.
+
+        May throw getmailOperationError if the server refuses the IDLE setup
+        (e.g. if the server does not support IDLE)
+
+        Default timeout is the maximum possible idle time according to RFC 2177.
+        """
+
+        if not self.supports_idle:
+            self.log.warning('IDLE not supported, so not idling')
+            raise getmailOperationError(
+                'IMAP4 IDLE requested, but not supported by server'
+            )
+
+
+        if self.SSL:
+            sock = self.conn.ssl()
+        else:
+            sock = self.conn.socket()
+
+        # Based on current imaplib IDLE patch: http://bugs.python.org/issue11245
+        self.conn.untagged_responses = {}
+        self.conn.select(folder)
+        tag = self.conn._command('IDLE')
+        data = self.conn._get_response() # read continuation response
+
+        if data is not None:
+            raise getmailOperationError(
+                'IMAP4 IDLE requested, but server refused IDLE request: %s' 
+                % data
+            )
+
+        self.log.debug('Entering IDLE mode (server says "%s")\n' 
+                       % self.conn.continuation_response)
+
+        try:
+            aborted = None
+            (readable, unused, unused) = select.select([sock], [], [], timeout)
+        except KeyboardInterrupt, o:
+            # Delay raising this until we've stopped IDLE mode
+            aborted = o
+
+        if aborted is not None:
+            self.log.debug('IDLE mode cancelled\n')
+        elif readable:
+            # The socket has data waiting; server has updated status
+            self.log.info('IDLE message received\n')
+        else:
+            self.log.debug('IDLE timeout (%ds)\n' % timeout)
+
+        try:
+            self.conn.untagged_responses = {}
+            self.conn.send('DONE\r\n')
+            self.conn._command_complete('IDLE', tag)
+        except imaplib.IMAP4.error, o:
+            return False
+
+        if aborted:
+            raise aborted
+
+        return True
+
     def quit(self):
         self.log.trace()
-        if not getattr(self, 'conn', None):
+        if not self.conn:
             return
         try:
             if self.mailbox_selected is not False:
